#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QSqlQuery>
#include <QSqlRecord>

#include <QProcess>
#include <QFileDialog>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    db = QSqlDatabase::addDatabase("QSQLITE");
    db.setDatabaseName("test.sqlite");
    db.open ();

    table = "colors";

    model = new QSqlTableModel (this, db);
    model->setTable (table);
    model->select ();
    ui->tableView->setModel (model);

    load ("example.txt");
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::load (QString file_name)
{
    QFile file (file_name);
    if (file.open (QIODevice::ReadOnly | QIODevice::Text))
    {
        QByteArray a = file.readAll ();
        // QString s = QString::fromLatin1(a.data ());
        ui->input->setPlainText (a);
    }
}

void MainWindow::put (QString s)
{
    if (!silent)
       answer += s;
}

void MainWindow::scan()
{
    answer = "";
    silent = false;

    QSqlQuery query = db.exec("SELECT * FROM " + table);

    QString txt = ui->input->toPlainText();
    int len = txt.length();
    int inx = 0;

    int start = 0;

    while (inx < len)
    {
        QChar c = txt [inx ++];
        if (c == '\r')
           /* nothing */ ;
        else if (c != '@')
           put (c);
        else
        {
           QString id = "";
           bool stop = false;
           while (inx < len && ! stop)
           {
              c = txt [inx];
              if (c.isLetter())
              {
                  id += c;
                  inx ++;
              }
              else stop = true;
           }
           if (id == "")
              put ("@");
           else if (id == "repeat")
           {
              start = inx;
              bool ok = query.next ();
              silent = ! ok;
           }
           else if (id == "next")
           {
              bool ok = query.next ();
              if (ok)
                 inx = start;
              else
                 silent = false;
           }
           else
           {
               QVariant v = query.value (id);
               QString s = v.isValid() ? v.toString() :
                           "[unknown field " + id + "]";

               // QString s = query.value (id).toString();
               // if (! query.record().contains(id))
               //    s = "[unknown field " + id + "]";
               put (s);
           }
        }
    }
    ui->output->setPlainText(answer);
}

void MainWindow::on_actionRun_triggered()
{
    scan ();

    QFile file ("output.txt");
    if (file.open (QIODevice::WriteOnly | QIODevice::Text))
    {
        QString s = ui->output->toPlainText();
        QByteArray a = s.toLatin1 ();
        file.write (a);
        file.close ();
    }

    QProcess proc (this);
    QStringList args;
    args << "output.txt";
    proc.execute ("pluma", args);
}

void MainWindow::on_actionOpen_triggered()
{
    QString s = QFileDialog::getOpenFileName (this, "File with template");
    if (s != "")
       load (s);
}
